--------------------------------------------------------------------------------
-- File       : gig_eth_pcs_pma_v11_1_block.vhd
-- Author     : Xilinx Inc.
--------------------------------------------------------------------------------
-- (c) Copyright 2009 Xilinx, Inc. All rights reserved.
--
-- This file contains confidential and proprietary information
-- of Xilinx, Inc. and is protected under U.S. and
-- international copyright and other intellectual property
-- laws.
--
-- DISCLAIMER
-- This disclaimer is not a license and does not grant any
-- rights to the materials distributed herewith. Except as
-- otherwise provided in a valid license issued to you by
-- Xilinx, and to the maximum extent permitted by applicable
-- law: (1) THESE MATERIALS ARE MADE AVAILABLE "AS IS" AND
-- WITH ALL FAULTS, AND XILINX HEREBY DISCLAIMS ALL WARRANTIES
-- AND CONDITIONS, EXPRESS, IMPLIED, OR STATUTORY, INCLUDING
-- BUT NOT LIMITED TO WARRANTIES OF MERCHANTABILITY, NON-
-- INFRINGEMENT, OR FITNESS FOR ANY PARTICULAR PURPOSE; and
-- (2) Xilinx shall not be liable (whether in contract or tort,
-- including negligence, or under any other theory of
-- liability) for any loss or damage of any kind or nature
-- related to, arising under or in connection with these
-- materials, including for any direct, or any indirect,
-- special, incidental, or consequential loss or damage
-- (including loss of data, profits, goodwill, or any type of
-- loss or damage suffered as a result of any action brought
-- by a third party) even if such damage or loss was
-- reasonably foreseeable or Xilinx had been advised of the
-- possibility of the same.
--
-- CRITICAL APPLICATIONS
-- Xilinx products are not designed or intended to be fail-
-- safe, or for use in any application requiring fail-safe
-- performance, such as life-support or safety devices or
-- systems, Class III medical devices, nuclear facilities,
-- applications related to the deployment of airbags, or any
-- other applications that could lead to death, personal
-- injury, or severe property or environmental damage
-- (individually and collectively, "Critical
-- Applications"). Customer assumes the sole risk and
-- liability of any use of Xilinx products in Critical
-- Applications, subject only to applicable laws and
-- regulations governing limitations on product liability.
--
-- THIS COPYRIGHT NOTICE AND DISCLAIMER MUST BE RETAINED AS
-- PART OF THIS FILE AT ALL TIMES. 
-- 
-- 
--------------------------------------------------------------------------------
-- Description: This Core Block Level wrapper connects the core to a
--              Spartan-6 tranceiver.
--
--              All of the clock circuitry required for a single
--              instance of the core is included.
--
--
--   ------------------------------------------------------------
--   |                                                          |
--   |        ------------------          -----------------     |
--   |        |      Core      |          |   tranceiver  |     |
--   |        |                |          |               |     |
--   |        |                |          |               |     |
--   |        |                |          |               |     |
-- ---------->| GMII           |--------->|           TXP |-------->
--   |        | Tx          Tx |          |           TXN |     |
--   |        |                |          |               |     |
--   |        |                |tranceiver|               |     |
--   |        |                |    I/F   |               |     |
--   |        |                |          |               |     |
--   |        |                |          |               |     |
--   |        | GMII           |          |           RXP |     |
-- <----------| Rx          Rx |<---------|           RXN |<--------
--   |        |                |          |               |     |
--   |        ------------------          -----------------     |
--   |                                                          |
--   ------------------------------------------------------------
--
--


library unisim;
use unisim.vcomponents.all;

library ieee;
use ieee.std_logic_1164.all;


--------------------------------------------------------------------------------
-- The entity declaration for the Core Block Level wrapper.
--------------------------------------------------------------------------------

entity gig_eth_pcs_pma_v11_1_block is
      generic (
      -- Set to 1 to Speed up the GTP simulation
      SIM_GTPRESET_SPEEDUP : integer   := 0
      );
      port(
      gtpclkout            : out std_logic;                    -- tranceiver output clock made available to the FPGA fabric.


      --------------------------------------------------------------------------
      -- Core connected to GTP1
      --------------------------------------------------------------------------

      gtpreset1            : in  std_logic;                    -- Full System GTP Reset

      -- GMII Interface
      -----------------
      gmii_txd1            : in std_logic_vector(7 downto 0);  -- Transmit data from client MAC.
      gmii_tx_en1          : in std_logic;                     -- Transmit control signal from client MAC.
      gmii_tx_er1          : in std_logic;                     -- Transmit control signal from client MAC.
      gmii_rxd1            : out std_logic_vector(7 downto 0); -- Received Data to client MAC.
      gmii_rx_dv1          : out std_logic;                    -- Received control signal to client MAC.
      gmii_rx_er1          : out std_logic;                    -- Received control signal to client MAC.
      gmii_isolate1        : out std_logic;                    -- Tristate control to electrically isolate GMII.

      -- Management: Alternative to MDIO Interface
      --------------------------------------------
      configuration_vector1: in std_logic_vector(3 downto 0);  -- Alternative to MDIO interface.

      status_vector1       : out std_logic_vector(15 downto 0); -- Core status.
      reset1               : in std_logic;                     -- Asynchronous reset for entire core.
      signal_detect1       : in std_logic;                     -- Input from PMD to indicate presence of optical input.

      --------------------------------------------------------------------------
      -- Tranceiver interfaces
      --------------------------------------------------------------------------
      clkin                : in std_logic;                     -- tranceiver 125MHz clock, very high quality.
      userclk2             : in std_logic;                     -- 125MHz reference clock for all core logic..

      txp0                 : out std_logic;                    -- Differential +ve of serial transmission from PMA to PMD.
      txn0                 : out std_logic;                    -- Differential -ve of serial transmission from PMA to PMD.
      rxp0                 : in std_logic;                     -- Differential +ve for serial reception from PMD to PMA.
      rxn0                 : in std_logic;                     -- Differential -ve for serial reception from PMD to PMA.

      txp1                 : out std_logic;                    -- Differential +ve of serial transmission from PMA to PMD.
      txn1                 : out std_logic;                    -- Differential -ve of serial transmission from PMA to PMD.
      rxp1                 : in std_logic;                     -- Differential +ve for serial reception from PMD to PMA.
      rxn1                 : in std_logic                      -- Differential -ve for serial reception from PMD to PMA.

      );
end gig_eth_pcs_pma_v11_1_block;



architecture block_level of gig_eth_pcs_pma_v11_1_block is



   -----------------------------------------------------------------------------
   -- Component Declaration for the Transceiver wrapper
   -----------------------------------------------------------------------------

   component transceiver_B
   generic (
      -- Set to 1 to Speed up the GTP simulation
      SIM_GTPRESET_SPEEDUP : integer   := 0
   );
   port (
      gtpclkout            : out std_logic;
      gtpclkin             : in  std_logic;

      gtpreset1            : in  std_logic;
      plllkdet1            : out std_logic;
      resetdone1           : out std_logic;
      enablealign1         : in  std_logic;
      powerdown1           : in  std_logic;
      loopback1            : in  std_logic;
      rxchariscomma1       : out std_logic;
      rxcharisk1           : out std_logic;
      rxclkcorcnt1         : out std_logic_vector (2 downto 0);
      rxdata1              : out std_logic_vector (7 downto 0);
      rxdisperr1           : out std_logic;
      rxnotintable1        : out std_logic;
      rxrundisp1           : out std_logic;
      rxbuferr1            : out std_logic;
      rxusrclk1            : in  std_logic;
      rxusrclk21           : in  std_logic;
      rxreset1             : in  std_logic;
      txchardispmode1      : in  std_logic;
      txchardispval1       : in  std_logic;
      txcharisk1           : in  std_logic;
      txdata1              : in  std_logic_vector (7 downto 0);
      txbuferr1            : out std_logic;
      txusrclk1            : in  std_logic;
      txusrclk21           : in  std_logic;
      txreset1             : in  std_logic;

      txn0                 : out std_logic;
      txp0                 : out std_logic;
      rxn0                 : in  std_logic;
      rxp0                 : in  std_logic;

      txn1                 : out std_logic;
      txp1                 : out std_logic;
      rxn1                 : in  std_logic;
      rxp1                 : in  std_logic;

      clkin                : in  std_logic
          );
   end component;



   -----------------------------------------------------------------------------
   -- Component Declaration for the 1000BASE-X PCS/PMA sublayer core.
   -----------------------------------------------------------------------------
   component gig_eth_pcs_pma_v11_1
      port(

      -- Core <=> Tranceiver Interface
      --------------------------------

      mgt_rx_reset         : out std_logic;                    -- tranceiver connection: reset for the receiver half of the tranceiver
      mgt_tx_reset         : out std_logic;                    -- tranceiver connection: reset for the transmitter half of the tranceiver
      userclk              : in std_logic;                     -- Routed to TXUSERCLK and RXUSERCLK of Transceiver.
      userclk2             : in std_logic;                     -- Routed to TXUSERCLK2 and RXUSERCLK2 of Transceiver.
      dcm_locked           : in std_logic;                     -- LOCKED signal from DCM.

      rxbufstatus          : in std_logic_vector (1 downto 0); -- tranceiver connection: Elastic Buffer Status.
      rxchariscomma        : in std_logic;                     -- tranceiver connection: Comma detected in RXDATA.
      rxcharisk            : in std_logic;                     -- tranceiver connection: K character received (or extra data bit) in RXDATA.
      rxclkcorcnt          : in std_logic_vector(2 downto 0);  -- tranceiver connection: Indicates clock correction.
      rxdata               : in std_logic_vector(7 downto 0);  -- tranceiver connection: Data after 8B/10B decoding.
      rxdisperr            : in std_logic;                     -- tranceiver connection: Disparity-error in RXDATA.
      rxnotintable         : in std_logic;                     -- tranceiver connection: Non-existent 8B/10 code indicated.
      rxrundisp            : in std_logic;                     -- tranceiver connection: Running Disparity of RXDATA (or extra data bit).
      txbuferr             : in std_logic;                     -- tranceiver connection: TX Buffer error (overflow or underflow).

      powerdown            : out std_logic;                    -- tranceiver connection: Powerdown the tranceiver
      txchardispmode       : out std_logic;                    -- tranceiver connection: Set running disparity for current byte.
      txchardispval        : out std_logic;                    -- tranceiver connection: Set running disparity value.
      txcharisk            : out std_logic;                    -- tranceiver connection: K character transmitted in TXDATA.
      txdata               : out std_logic_vector(7 downto 0); -- tranceiver connection: Data for 8B/10B encoding.
      enablealign          : out std_logic;                    -- Allow the transceivers to serially realign to a comma character.

      -- GMII Interface
      -----------------

      gmii_txd             : in std_logic_vector(7 downto 0);  -- Transmit data from client MAC.
      gmii_tx_en           : in std_logic;                     -- Transmit control signal from client MAC.
      gmii_tx_er           : in std_logic;                     -- Transmit control signal from client MAC.
      gmii_rxd             : out std_logic_vector(7 downto 0); -- Received Data to client MAC.
      gmii_rx_dv           : out std_logic;                    -- Received control signal to client MAC.
      gmii_rx_er           : out std_logic;                    -- Received control signal to client MAC.
      gmii_isolate         : out std_logic;                    -- Tristate control to electrically isolate GMII.

      -- Management: Alternative to MDIO Interface
      --------------------------------------------
      configuration_vector : in std_logic_vector(3 downto 0);

      status_vector        : out std_logic_vector(15 downto 0); -- Core status.
      reset                : in std_logic;                     -- Asynchronous reset for entire core.
      signal_detect        : in std_logic                      -- Input from PMD to indicate presence of optical input.
      );

   end component;



  ------------------------------------------------------------------------------
  -- internal signals used in this Block level wrapper.
  ------------------------------------------------------------------------------

  -- Core <=> tranceiver (GTP1) interconnect
  signal plllkdet1         : std_logic;                        -- GTP1 PLL Lock indication
  signal mgt_rx_reset1     : std_logic;                        -- Reset for the receiver half of the tranceiver
  signal mgt_tx_reset1     : std_logic;                        -- Reset for the transmitter half of the tranceiver
  signal rxbufstatus1      : std_logic_vector (1 downto 0);    -- Elastic Buffer Status (bit 1 asserted indicates overflow or underflow).
  signal rxchariscomma1    : std_logic;                        -- Comma detected in RXDATA.
  signal rxcharisk1        : std_logic;                        -- K character received (or extra data bit) in RXDATA.
  signal rxclkcorcnt1      : std_logic_vector(2 downto 0);     -- Indicates clock correction.
  signal rxdata1           : std_logic_vector(7 downto 0);     -- Data after 8B/10B decoding.
  signal rxdisperr1        : std_logic;                        -- Disparity-error in RXDATA.
  signal rxnotintable1     : std_logic;                        -- Non-existent 8B/10 code indicated.
  signal rxrundisp1        : std_logic;                        -- Running Disparity after current byte, becomes 9th data bit when RXNOTINTABLE='1'.
  signal txbuferr1         : std_logic;                        -- TX Buffer error (overflow or underflow).
  signal powerdown1        : std_logic;                        -- Powerdown the tranceiver
  signal txchardispmode1   : std_logic;                        -- Set running disparity for current byte.
  signal txchardispval1    : std_logic;                        -- Set running disparity value.
  signal txcharisk1        : std_logic;                        -- K character transmitted in TXDATA.
  signal txdata1           : std_logic_vector(7 downto 0);     -- Data for 8B/10B encoding.
  signal enablealign1      : std_logic;                        -- Allow the transceivers to serially realign to a comma character.

  signal loopback          : std_logic;                        -- Set the tranceiver for loopback.


begin



  ------------------------------------------------------------------------------
  -- Instantiate the core connected to GTP1
  ------------------------------------------------------------------------------

  gig_eth_pcs_pma_core_1 : gig_eth_pcs_pma_v11_1
    port map (
      mgt_rx_reset         => mgt_rx_reset1,
      mgt_tx_reset         => mgt_tx_reset1,
      userclk              => userclk2,
      userclk2             => userclk2,
      dcm_locked           => plllkdet1,
      rxbufstatus          => rxbufstatus1,
      rxchariscomma        => rxchariscomma1,
      rxcharisk            => rxcharisk1,
      rxclkcorcnt          => rxclkcorcnt1,
      rxdata               => rxdata1,
      rxdisperr            => rxdisperr1,
      rxnotintable         => rxnotintable1,
      rxrundisp            => rxrundisp1,
      txbuferr             => txbuferr1,
      powerdown            => powerdown1,
      txchardispmode       => txchardispmode1,
      txchardispval        => txchardispval1,
      txcharisk            => txcharisk1,
      txdata               => txdata1,
      enablealign          => enablealign1,
      gmii_txd             => gmii_txd1,
      gmii_tx_en           => gmii_tx_en1,
      gmii_tx_er           => gmii_tx_er1,
      gmii_rxd             => gmii_rxd1,
      gmii_rx_dv           => gmii_rx_dv1,
      gmii_rx_er           => gmii_rx_er1,
      gmii_isolate         => gmii_isolate1,
      configuration_vector => configuration_vector1,
      status_vector        => status_vector1,
      reset                => reset1,
      signal_detect        => signal_detect1

      );

   rxbufstatus1(0) <= '0';



   -----------------------------------------------------------------------------
   -- Component Instantiation for the Spartan-6 Transceiver wrapper
   -----------------------------------------------------------------------------


   transceiver_inst : transceiver_B
   generic map
   (
      -- Simulation attribute
      SIM_GTPRESET_SPEEDUP => SIM_GTPRESET_SPEEDUP
   )
   port map (
      gtpclkout            => gtpclkout,
      gtpclkin             => userclk2,

      -- tranceiver 1
      gtpreset1            => gtpreset1,
      plllkdet1            => plllkdet1,
      resetdone1           => open,
      enablealign1         => enablealign1,
      powerdown1           => powerdown1,
      loopback1            => loopback,
      rxchariscomma1       => rxchariscomma1,
      rxcharisk1           => rxcharisk1,
      rxclkcorcnt1         => rxclkcorcnt1,
      rxdata1              => rxdata1,
      rxdisperr1           => rxdisperr1,
      rxnotintable1        => rxnotintable1,
      rxrundisp1           => rxrundisp1,
      rxbuferr1            => rxbufstatus1(1),
      rxusrclk1            => userclk2,
      rxusrclk21           => userclk2,
      rxreset1             => mgt_rx_reset1,
      txchardispmode1      => txchardispmode1,
      txchardispval1       => txchardispval1,
      txcharisk1           => txcharisk1,
      txdata1              => txdata1,
      txbuferr1            => txbuferr1,
      txusrclk1            => userclk2,
      txusrclk21           => userclk2,
      txreset1             => mgt_tx_reset1,

      txn0                 => txn0,
      txp0                 => txp0,
      rxn0                 => rxn0,
      rxp0                 => rxp0,

      txn1                 => txn1,
      txp1                 => txp1,
      rxn1                 => rxn1,
      rxp1                 => rxp1,

      clkin                => clkin
   );


   -- Loopback is performed in the core itself.  To alternatively use
   -- tranceiver loopback, please drive this port appropriately.
   loopback <= '0';



end block_level;

